function [u thebigmatrix rhs]=mini(xn,yn,element,dirichlet_flag,dirichlet_values,mu,f)
% Solves Stokes equation with MINI finite elements
% element[nel,3], x[n], y[n]: define mesh
% mu=viscosity
% u=[ul p]
% ul[2,n] is the velocity
% p[n] is the pressure
% f[2,nel] is the function f evaluated in (the centroid of) every triangle
% dirichlet_flag[n,3],dirichlet_values[n,3]
% nodes j with dirichlet_flag(i,j)=1 have the j-th boundary condition set
% to dirichlet_values[i,j]

rot=[0 -1;1 0]; %rotation of 90° in the positive direction -- needed to get the gradients from the sides of a triangle

xn=xn(:);
yn=yn(:);
n=size(xn,1);
nel=size(element,1);
if (size(element,2)~=3)
    error('wrong size for the element matrix');
end

element=transpose(element); %really, element should always be memorized transposed, since it is only useful when accessed row-wise

%%the matrix to assemble is
% [A   0  B1] ul1    ff1
% [0   A  B2] ul2 =  ff2 = rhs
% [B1' B2' C]  p      D
%
% the two A's are relative to different components of the velocity u and
% therefore do not mix
% take care, b(u,p)=-\int (\div u) p --  with a minus sign!
% C and D derive from bubble condensation
% C

%% instead of filling in the matrices directly (which would take O(n^2)) we
%% memorize the 9*nel values to fill in in an array and feed it to sparse() -- Canciani's idea
%% saves time, but consumes memory, since Matlab allocates a sparse matrix
%% with size 9*nel by default

%allocates empty structures
rhs=zeros(n,3);
spA_val=zeros(9,nel);
spB1_val=zeros(9,nel);
spB2_val=zeros(9,nel);
spC_val=zeros(9,nel);

%%assembles the matrix blocks and right-hand side
%%accumulates the integrals over every T \in triangulation
for j=1:nel
    %defines the simple version of the bubble condensation
    alphaT=1;
    %%coordinates of the vertices: x(:,1) is the first vertex, x(:,2) is the second one...
    x=[xn(element(:,j)) yn(element(:,j))]';
    area=.5*det([x; ones(1,3)]);
    if(area<0) error('wrong orientation!'); end
    %hT should be a "characteristic length" of the element -- here we
    %take the semiperimeter
    hT=.5*(norm(x(:,1)-x(:,2))+norm(x(:,2)-x(:,3))+norm(x(:,3)-x(:,1)));
    %%sides dw: dw(:,1) = e3-e2, and cyclic
    dw=x(:,[3 1 2])-x(:,[2 3 1]);
    %let phi_i be the basis function such that phi_i(vertex_j)=\delta_ij
    %gradientsA(i,:) = \nabla \phi_i * area(T)
    %take care, they are row vectors
    gradientsA=.5*(rot*dw)';
    %%calculates a(ij) = .25*dw_i*dw_j/area and accumulates in the A matrix
    %A(element(:,j),element(:,j))=A(element(:,j),element(:,j))+.25*dw'*dw/area;
    int=gradientsA*gradientsA'/area;
    spA_val(:,j)=mu*int(:);
    %C(i,j)= h_T^2 \alpha_T \int \nabla p \cdot \nabla q
    %C is the same as A by def + scale factors
    spC_val(:,j)=-hT^2*alphaT*int(:);
    %%calculates B1(ij),B2(ij)
    %B1_ij=1/6*dw(2,i)
    %B2_ij=-1/6*dw(1,i)
    spB1_val(:,j)=repmat(-1/3*gradientsA(:,1),3,1);    %vector voodoo for dw21 dw22 dw23 dw21 dw22 dw23 dw21 dw22 dw23
    spB2_val(:,j)=repmat(-1/3*gradientsA(:,2),3,1);    %vector voodoo for dw21 dw22 dw23 dw21 dw22 dw23 dw21 dw22 dw23
    %%ff1, ff2: evals integral <fv> over T
    rhs(element(:,j),1) = rhs(element(:,j),1) + 1/3*area*f(1,j);
    rhs(element(:,j),2) = rhs(element(:,j),2) + 1/3*area*f(2,j);
    %%D=-\int f \cdot \nabla q
    rhs(element(:,j),3) = rhs(element(:,j),3)-hT^2* alphaT*gradientsA*f(:,j);
end

%%builds the indices vectors sp_x and sp_y for sparse() -- uses some vector voodoo
sp_y=repmat(element,3,1);
sp_x=element([1 1 1 2 2 2 3 3 3],:);

%%builds matrices
A=sparse(sp_x(:),sp_y(:),spA_val,n,n);
B1=sparse(sp_x(:),sp_y(:),spB1_val,n,n);
B2=sparse(sp_x(:),sp_y(:),spB2_val,n,n);
C=sparse(sp_x(:),sp_y(:),spC_val,n,n);

thebigmatrix=[A sparse(n,n) B1;sparse(n,n) A B2; B1' B2' C];
rhs=rhs(:);

dirichlet_flag=dirichlet_flag(:);
dirichlet_values=dirichlet_values(:);

%%Imposes dirichlet's conditions
for j=1:3*n %FIXME:vectorize
    if(dirichlet_flag(j))
        thebigmatrix(j,:)=0; %this should not fill up the row (tested with Matlab 6.0)
        thebigmatrix(j,j)=1;
        rhs(j)=dirichlet_values(j);
    end
end

%%solves the resulting system
[L1,U1] = luinc(thebigmatrix,1e-5);
[u,flag] = gmres(thebigmatrix,rhs+1.e-16); %FIXME: gmres seems bugged, here we must add 1.e-16 to rhs to avoid a pesky error
%u=thebigmatrix\rhs;
u=reshape(u,n,3);