function [u, thebigmatrix, rhs]=mini(xn,yn,element,dirichlet_flag,dirichlet_values,mu,f,alphamode,solmode)
% Solves Stokes equation with MINI finite elements
% element[nel,3], x[n], y[n]: define mesh
% mu=viscosity
% u=[ul p]
% ul[2,n] is the velocity
% p[n] is the pressure
% f[2,nel] is the function f evaluated in (the centroid of) every triangle
% dirichlet_flag[n,1],dirichlet_values[n,2]
% nodes j with dirichlet_flag(j)=1 have the velocity boundary cond set to
% dirichlet_values(j,:)
% alphamode = nocond - exact - approximate : without condensation, condensation with exact value of alpha_T, condensation+approximation
% solmode = gmres - lu : how to solve the system

rot=[0 -1;1 0]; %rotation of 90° in the positive direction -- needed to get the gradients from the sides of a triangle

xn=xn(:);
yn=yn(:);
n=size(xn,1);
nel=size(element,1);
if (size(element,2)~=3)
    error('wrong size for the element matrix');
end

element=transpose(element); %really, element should always be memorized transposed, since it is only useful when accessed row-wise

%%the matrix to assemble is
% [A   0  B1] ul1    fv1
% [0   A  B2] ul2 =  fv2 = rhs
% [B1' B2' C]  p     fpress
%
% the two A's are relative to different components of the velocity u and
% therefore do not mix
% take care, b(u,p)=-\int (\div u) p --  with a minus sign!
% C and D derive from bubble condensation
% A, B1, B2, C have the same sparsity pattern

% in the nocond case:
% A  0  0  0  B1
% 0  A  0  0  B2
% 0  0  AT 0  0
% 0  0  0  AT BT
% B1 B2 0  BT BT
% AT is diagonal
% A, B1, B2 are the same as above

%% instead of filling in the matrices directly (which would take O(n^2)) we
%% memorize the 9*nel values to fill in in an array and feed it to sparse() -- Canciani's idea
%% saves time, but consumes memory, since Matlab allocates a sparse matrix
%% with size 9*nel by default, and I found no trick to make it shrink
A_values=zeros(9,nel);
B1_values=zeros(9,nel);
B2_values=zeros(9,nel);

fv1=zeros(n,1);
fv2=zeros(n,1);

if(strcmp(alphamode,'nocond'))
    %only for mode without condensation
    AT=zeros(nel,1); %its diagonal actually
    BT_values=zeros(6,nel);
    BT=sparse(2*nel,n);
else
    %only for condensation
    C_values=zeros(9,nel);
    C=sparse(n,n);
    fpress=zeros(n,1);
end

%%assembles the matrix blocks and right-hand side
%%accumulates the integrals over every T \in triangulation
for j=1:nel
    %%coordinates of the vertices: x(:,1) is the first vertex, x(:,2) is the second one...
    x=[xn(element(:,j)) yn(element(:,j))]';
    area=.5*det([x; ones(1,3)]);
    if(area<0) error('wrong orientation!'); end

    %%sides dw: dw(:,1) = e3-e2, and cyclic
    dw=x(:,[3 1 2])-x(:,[2 3 1]);
    %let phi_i be the basis function such that phi_i(vertex_j)=\delta_ij
    %gradientsA(i,:) = \nabla \phi_i * area(T)
    %take care, they are row vectors
    gradientsA=.5*(rot*dw)';
    %%calculates a(ij) = .25*dw_i*dw_j/area and accumulates in the A matrix
    %A(element(:,j),element(:,j))=A(element(:,j),element(:,j))+.25*dw'*dw/area;
    int=gradientsA*gradientsA'/area;
    A_values(:,j)=mu*int(:);
    %%calculates B1(ij),B2(ij)
    B1_values(:,j)=repmat(-1/3*gradientsA(:,1),3,1);
    B2_values(:,j)=repmat(-1/3*gradientsA(:,2),3,1);
    
    %%fv1, fv2: evals integral <fv> over T
    fv1(element(:,j)) = fv1(element(:,j)) + 1/3*area*f(1,j);
    fv2(element(:,j)) = fv2(element(:,j)) + 1/3*area*f(2,j);
    %switch over bubble cond mode
    if(strcmp(alphamode,'nocond'))
        %strangeint is area(T)*|bubble|_{H^1}^2 (the denominator of alpha_T
        strangeint=(gradientsA(:,1))'*gradientsA(:,1)+(gradientsA(:,2))'*gradientsA(:,2);
        AT(j)=mu*strangeint/area/180;
        %BT([j nel+j],element(:,j))=1/60*gradientsA';
        BT_values(:,j)=1/60*gradientsA(:);
    elseif(strcmp(alphamode,'exact'))
        strangeint=(gradientsA(:,1))'*gradientsA(:,1)+(gradientsA(:,2))'*gradientsA(:,2);
        alphaT=1/20*area^2/mu/strangeint;

	% uncomment to print the computed value of alphaT at each iteration
        % hT=.5*(norm(x(:,1)-x(:,2))+norm(x(:,2)-x(:,3))+norm(x(:,3)-x(:,1)));
	% alphaT

        %C is the same as A by def + scale factors
        %C(i,j)= h_T^2 \alpha_T \int \nabla p \cdot \nabla q
        %C(element(:,j),element(:,j)) = C(element(:,j),element(:,j))-alphaT*int;
        C_values(:,j)=-alphaT*int(:);
        fpress(element(:,j)) = fpress(element(:,j))-alphaT*gradientsA*f(:,j);
    elseif(strcmp(alphamode,'approximate'))
        %alpha_T=1
        %hT should be a "characteristic length" of the element -- here we
        %take the semiperimeter
        hT=.5*(norm(x(:,1)-x(:,2))+norm(x(:,2)-x(:,3))+norm(x(:,3)-x(:,1)));
        %C is the same as A by def + scale factors
        %C(i,j)= h_T^2 \alpha_T \int \nabla p \cdot \nabla q
        %C(element(:,j),element(:,j))=C(element(:,j),element(:,j))-hT^2*int;
        C_values(:,j)=-hT^2*int(:);
        fpress(element(:,j)) = fpress(element(:,j))-hT^2*gradientsA*f(:,j);
    else
        error('Wrong alpha mode');
    end
end

%%fills the sparse matrices
%%builds the indices vectors sp_x and sp_y for sparse() -- uses some bad vector voodoo
sp_x=repmat(element,3,1);
sp_y=element([1 1 1 2 2 2 3 3 3],:);

A=sparse(sp_x(:),sp_y(:),A_values,n,n);
B1=sparse(sp_x(:),sp_y(:),B1_values,n,n);
B2=sparse(sp_x(:),sp_y(:),B2_values,n,n);
if(strcmp(alphamode,'nocond'))
    sp_x=[repmat(1:nel,3,1) ; repmat(nel+1:2*nel,3,1)];
    sp_y=repmat(element,2,1);
    BT=sparse(sp_x(:),sp_y(:),BT_values,2*nel,n);
else
    C=sparse(sp_x(:),sp_y(:),C_values,n,n);
end

%%assembles the blocks
if(strcmp(alphamode,'nocond'))
    thebigmatrix=sparse(3*n+2*nel,3*n+2*nel);
    thebigmatrix(1:n,1:n)=A;
    thebigmatrix(n+1:2*n,n+1:2*n)=A;
    thebigmatrix(1:2*n,2*n+1:3*n)=[B1; B2];
    thebigmatrix(2*n+1:3*n,1:2*n)=[B1' B2'];
    thebigmatrix(3*n+1:3*n+2*nel,2*n+1:3*n)=BT;
    thebigmatrix(2*n+1:3*n,3*n+1:3*n+2*nel)=BT';
    thebigmatrix(3*n+1:3*n+2*nel,3*n+1:3*n+2*nel)=spdiags([AT;AT],0,2*nel,2*nel);
    rhs=[fv1;fv2;zeros(n+2*nel,1)];
else
    thebigmatrix=[A sparse(n,n) B1;sparse(n,n) A B2; B1' B2' C];
    rhs=[fv1;fv2;fpress];
end

%%Imposes dirichlet's conditions
for j=1:n %FIXME:vectorize
    if(dirichlet_flag(j))
        thebigmatrix([j n+j],:)=0; %this should not fill up the row (tested with Matlab 6.0)
        thebigmatrix(j,j)=1;
        thebigmatrix(n+j,n+j)=1;
        rhs([j n+j])=dirichlet_values(j,:);
    end
end

%%Imposes the value of p (pressure) in one point
thebigmatrix(2*n+1,:)=0;
thebigmatrix(2*n+1,2*n+1)=1;
rhs(2*n+1)=0;

m=size(thebigmatrix,1);

%%solves the resulting system
if(strcmp(solmode,'gmres'))
    [L1,U1] = luinc(thebigmatrix,1e-5);
    [u,flag] = gmres(thebigmatrix,rhs,5,1e-6,5,L1,U1);
elseif(strcmp(solmode,'modgmres'))
    [L1,U1] = luinc(thebigmatrix,1.e-4);
    u=gmres(thebigmatrix,rhs,5,1.e-6,5,L1,U1);
elseif(strcmp(solmode,'lu'))
    u=thebigmatrix\rhs;
else
    error('Wrong solmode');
end

%discards the bubble part of the solution if necessary
u=u(1:3*n);
u=reshape(u,n,3);