function [u, thebigmatrix, rhs]=mini_unopt(xn,yn,element,dirichlet_flag,dirichlet_values,mu,f,alphamode)
% Solves Stokes equation with MINI finite elements
% element[nel,3], x[n], y[n]: define mesh
% mu=viscosity
% u=[ul p]
% ul[2,n] is the velocity
% p[n] is the pressure
% f[2,nel] is the function f evaluated in (the centroid of) every triangle
% dirichlet_flag[n,1],dirichlet_values[n,2]
% nodes j with dirichlet_flag(j)=1 have the velocity boundary cond set to
% dirichlet_values(j,:)

rot=[0 -1;1 0]; %rotation of 90° in the positive direction -- needed to get the gradients from the sides of a triangle

xn=xn(:);
yn=yn(:);
n=size(xn,1);
nel=size(element,1);
if (size(element,2)~=3)
    error('wrong size for the element matrix');
end

element=transpose(element); %really, element should always be memorized transposed, since it is only useful when accessed row-wise

%%the matrix to assemble is
% [A   0  B1] ul1    fv1
% [0   A  B2] ul2 =  fv2 = rhs
% [B1' B2' C]  p     fpress
%
% the two A's are relative to different components of the velocity u and
% therefore do not mix
% take care, b(u,p)=-\int (\div u) p --  with a minus sign!
% C and D derive from bubble condensation
% C

%% instead of filling in the matrices directly (which would take O(n^2)) we
%% memorize the 9*nel values to fill in in an array and feed it to sparse() -- Canciani's idea
%% saves time, but consumes memory, since Matlab allocates a sparse matrix
%% with size 9*nel by default

%allocates empty structures
fv1=zeros(n,1);
fv2=zeros(n,1);
A=sparse(n,n);
B1=sparse(n,n);
B2=sparse(n,n);

if(strcmp(alphamode,'nocond'))
    %only for mode without cond
    AT=zeros(nel,1); %its diagonal actually
    BT=sparse(2*nel,n);
else
    %only for cond
    C=sparse(n,n);
    fpress=zeros(n,1);
end

%%assembles the matrix blocks and right-hand side
%%accumulates the integrals over every T \in triangulation
for j=1:nel
    %%coordinates of the vertices: x(:,1) is the first vertex, x(:,2) is the second one...
    x=[xn(element(:,j)) yn(element(:,j))]';
    area=.5*det([x; ones(1,3)]);
    if(area<0) error('wrong orientation!'); end

    %%sides dw: dw(:,1) = e3-e2, and cyclic
    dw=x(:,[3 1 2])-x(:,[2 3 1]);
    %let phi_i be the basis function such that phi_i(vertex_j)=\delta_ij
    %gradientsA(i,:) = \nabla \phi_i * area(T)
    %take care, they are row vectors
    gradientsA=.5*(rot*dw)';
    %%calculates a(ij) = .25*dw_i*dw_j/area and accumulates in the A matrix
    %A(element(:,j),element(:,j))=A(element(:,j),element(:,j))+.25*dw'*dw/area;
    int=gradientsA*gradientsA'/area;
    A(element(:,j),element(:,j))=A(element(:,j),element(:,j))+mu*int;
    %%calculates B1(ij),B2(ij)
    %B1_ij=1/6*dw(2,i)
    %B2_ij=-1/6*dw(1,i)
    B1(element(:,j),element(:,j))=B1(element(:,j),element(:,j))+repmat(-1/3*gradientsA(:,1),1,3);
    B2(element(:,j),element(:,j))=B2(element(:,j),element(:,j))+repmat(-1/3*gradientsA(:,2),1,3);
    %%fv1, fv2: evals integral <fv> over T
    fv1(element(:,j)) = fv1(element(:,j)) + 1/3*area*f(1,j);
    fv2(element(:,j)) = fv2(element(:,j)) + 1/3*area*f(2,j);
    %switch over bubble cond mode
    if(strcmp(alphamode,'nocond'))
        strangeint=(gradientsA(:,1))'*[2 1 1; 1 2 1; 1 1 2]*gradientsA(:,1)+(gradientsA(:,2))'*[2 1 1; 1 2 1; 1 1 2]*gradientsA(:,2);
        AT(j)=mu*strangeint/area/180;
        BT([j nel+j],element(:,j))=1/60*gradientsA';
    elseif(strcmp(alphamode,'exact'))
        strangeint=(gradientsA(:,1))'*[2 1 1; 1 2 1; 1 1 2]*gradientsA(:,1)+(gradientsA(:,2))'*[2 1 1; 1 2 1; 1 1 2]*gradientsA(:,2);
        alphaT=1/20*area^2/mu/strangeint;
        %C is the same as A by def + scale factors
        %C(i,j)= h_T^2 \alpha_T \int \nabla p \cdot \nabla q
        C(element(:,j),element(:,j)) = C(element(:,j),element(:,j))-alphaT*int;
        fpress(element(:,j)) = fpress(element(:,j))-alphaT*gradientsA*f(:,j);
    elseif(strcmp(alphamode,'approximate'))
        %alpha_T=1
        %hT should be a "characteristic length" of the element -- here we
        %take the semiperimeter
        hT=.5*(norm(x(:,1)-x(:,2))+norm(x(:,2)-x(:,3))+norm(x(:,3)-x(:,1)));
        %C is the same as A by def + scale factors
        %C(i,j)= h_T^2 \alpha_T \int \nabla p \cdot \nabla q
        C(element(:,j),element(:,j))=C(element(:,j),element(:,j))-hT^2*int;
        fpress(element(:,j)) = fpress(element(:,j))-hT^2*gradientsA*f(:,j);
    else
        error('Wrong alpha mode');
    end
end

%%assembles the blocks
if(strcmp(alphamode,'nocond'))
    thebigmatrix=sparse(3*n+2*nel,3*n+2*nel);
    thebigmatrix(1:n,1:n)=A;
    thebigmatrix(n+1:2*n,n+1:2*n)=A;
    thebigmatrix(1:2*n,2*n+1:3*n)=[B1; B2];
    thebigmatrix(2*n+1:3*n,1:2*n)=[B1' B2'];
    thebigmatrix(3*n+1:3*n+2*nel,2*n+1:3*n)=BT;
    thebigmatrix(2*n+1:3*n,3*n+1:3*n+2*nel)=BT';
    thebigmatrix(3*n+1:3*n+2*nel,3*n+1:3*n+2*nel)=spdiags([AT;AT],0,2*nel,2*nel);
    rhs=[fv1;fv2;zeros(n+2*nel,1)];
else
    thebigmatrix=[A sparse(n,n) B1;sparse(n,n) A B2; B1' B2' C];
    rhs=[fv1;fv2;fpress];
end

%%Imposes dirichlet's conditions
for j=1:n %FIXME:vectorize
    if(dirichlet_flag(j))
        thebigmatrix([j n+j],:)=0; %this should not fill up the row (tested with Matlab 6.0)
        thebigmatrix(j,j)=1;
        thebigmatrix(n+j,n+j)=1;
        rhs([j n+j])=dirichlet_values(j,:);
    end
end

%%Imposes value of p in one point
thebigmatrix(2*n+1,:)=0;
thebigmatrix(2*n+1,2*n+1)=1;
rhs(2*n+1)=0;

%%solves the resulting system
%[L1,U1] = luinc(thebigmatrix,1e-5);
%[u,flag] = gmres(thebigmatrix,rhs+1.e-16); %FIXME: gmres seems bugged, here we must add 1.e-16 to rhs to avoid an error
u=thebigmatrix\rhs;
u=u(1:3*n);
u=reshape(u,n,3);